/*================================================================
    BTW(Bak-Tang-Wiesenfeld) sandpile 
=================================================================*/    

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Graphic.h>
#include <XRS/Simple.h>
#include <XRS/Plane.h>
using namespace XRS;

#define PI         M_PI
#define Rand(zc)   int(drand48()*(zc))    // 0,,,zc-1 の乱数を出す


/************** 大域変数と関数宣言 ********************/

int   L,t;
int   **z,**z0,**z1;
int   zc,seed;        // パラメータ
int   ztotal,size;
int   *h;


void init(int);
void relax(int,int);
void boundary(void);
void draw(void); 


int main(int argc,char **argv)
{
   /*--------------- initialize ---------------------*/

   int ar=0;
   ar++; if(argc>ar) L   =atoi(argv[ar]); else L   =100;
   ar++; if(argc>ar) zc  =atoi(argv[ar]); else zc  =  5;
   ar++; if(argc>ar) seed=atoi(argv[ar]); else seed= 15;

   fprintf(stderr,"usage%% BTWsandplie [L] [zc] [seed]\n"
    " Current systemsize     =L   : L   =%d\n"
    " Threshhold of z        =zc  : zc  =%d\n"
    " Seed of random sequence=seed: seed=%d\n"
    ,L,zc,seed); 
   
   array(z ,L+1,L+1);
   array(z0,L+1,L+1);
   array(z1,L+1,L+1);
   h=new int [zc];

/*----- main routine --------*/
   
   init(0);

   int iadd,jadd;

   while(1){
      draw();

      iadd=Rand(L); jadd=Rand(L);
      if(iadd==0 || jadd==0) continue; 
      z[iadd][jadd]++;  
      relax(iadd,jadd);
      t++;
   }
}   


/***************** 初期条件 **************************/

void init(int)
{
  srand48(seed);

  for(int i=0; i<L+1; i++)
    for(int j=0; j<L+1; j++) z[i][j]=Rand(zc);

  t=0;
}


/***************** 緩和過程 **************************/

void relax(int iadd,int jadd)
{
  if(z[iadd][jadd]>=zc){
  
    for(int i=1; i<L; i++)
      for(int j=1; j<L; j++) z1[i][j]=z0[i][j]=z[i][j];

    int flag=1;

    size=0;  
    while(flag){
      flag=0;

      for(int i=1; i<L; i++)
        for(int j=1; j<L; j++)
          if(z[i][j]>=zc){
	    z0[i][j]-=4;   size++;  z1[i][j]=0; 
	    z0[i-1][j]++;  
	    z0[i+1][j]++;  
	    z0[i][j-1]++;  
	    z0[i][j+1]++;  
          } 

      for(int i=1; i<L; i++)
        for(int j=1; j<L; j++){
          z[i][j]=z0[i][j];
          if(z0[i][j]>=zc) flag=1;
        }
    }

    //printf("%d %d\n",t,size);
  }

  for(int i=0; i<zc; i++) h[i]=0;
 
  ztotal=0;
  for(int i=1; i<L; i++)
    for(int j=1; j<L; j++){ ztotal+=z[i][j]; h[z[i][j]]++;} 

  // printf("%d %d\n",t,ztotal);  
}



/***************** 描画 **************************/

Control_Window CW;
Graphic_Window Lattice;
Simple_Window  SW;
Plane_Window   PW;
GC             black=graphic_context("black");
GC             blue=graphic_context("blue");
int            Swidth=200, Sheight=200, Sdw; 
int            Pwidth=200, Pheight=200;
double         Phamp=0.2, Pwamp=1.0; 
void           PWticks(void);
double         Log10=log(10);
double         x0=0,y; 

#define SJ(x) int(Sheight*(0.9-double(x)/(L*L)))
#define PJ(x) int(-Phamp*(x)*Pheight)



void draw()
{
  static int initial=1;
  
  if(initial){
    /*----- initialize ----------*/ 
    CW<<init<<"initial"<<t<<"t"
      <<PARAMETER
      <<Pwamp<<"Pwamp"<<Phamp<<"Phamp";

    Lattice.geometry(L+1,L+1,4)<<"BTW sandpile"<<MONO(0.4,zc);
    SW.geometry(Swidth,Sheight)<<"distributoin of z";
    PW.geometry(Pwidth,Pheight)<<"size of avalanches";
    PWticks();
    initialize();
    initial=0;
  } 

  checkevent();
  if(CW.chev) PWticks();

  //================ スナップショット =====================

  Lattice.draw_2D(z1);


  //============== 状態変数 z の分布関数 ===================

  if(t%100==0){
    SW.clear();
    XDrawLine(SW.disp(),SW,blue,0,SJ(0),Swidth,SJ(0));

    Sdw=Swidth/zc;
    for(int i=0; i<zc; i++)
      XDrawLine(SW.disp(),SW,black,Sdw*i,SJ(0),Sdw*i,SJ(h[i]));
  }


  //================ 雪崩サイズの時系列 =====================

  if(size>0){
      y=log(double(size)/(L*L))/Log10;
      PW.draw_point(0,int(x0),PJ(y));
  }

  x0+=Pwamp;   if(x0>=double(Pwidth)){ x0=0.0; PW.clear(0); }

}




void PWticks(void)
{
     PW.clear();
     x0=0;
     y=log(1.0/(L*L))/Log10;
     PW.draw_line(2,0,PJ(0),Pwidth,PJ(0));
     PW.draw_line(1,0,PJ(-1.0),Pwidth,PJ(-1.0));
     PW.draw_line(1,0,PJ(-2.0),Pwidth,PJ(-2.0));
     PW.draw_line(1,0,PJ(-3.0),Pwidth,PJ(-3.0));
     PW.draw_line(1,0,PJ(-4.0),Pwidth,PJ(-4.0));
     PW.draw_line(1,0,PJ(-5.0),Pwidth,PJ(-5.0));
     PW.draw_line(2,0,PJ(y),Pwidth,PJ(y));
}
