/*=================================================================
    Frictional motion 
    $B0lDjB.EY(BV$B$G0\F0$9$kJ*BN$H%P%M$G$D$J$,$C$?J*BN$,Dq93NO(Bf$B$r<u$1$F(B
    $B9T$&1?F0!#(Bu: 2$BJ*BN$N5wN%!"(Bv: $B5wN%$N;~4VJQ2=N((B
      u'=v
      v'=-u+f(V-v)
      f(z)=b(1+cz^4)/(1+z^4)z
=================================================================*/

/*************************************************************
    $B%$%s%/%k!<%I$HBg0hJQ?t(B
*************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <time.h>
#include "include/ode.h"


double V=0.0,b=1.0,c=0.2;
double *xp,**yp,dxsav,xx=0.0,yy=0.0;
int    kmax,kount;
double f(double z){ 
  double z2=z*z; double z4=z2*z2;
  return b*z*(1.0+c*z4)/(1.0+z4); }


/*************************************************************
    $B%0%i%U%#%C%/MQ$N%$%s%/%k!<%I$HJQ?t!"4X?t$N@k8@(B
*************************************************************/

#include <XRS/Control.h>
#include <XRS/Plane.h>
using namespace XRS;

#define WinI(x)  (int)((double)width*(0.4+AmpV*(x)))
#define WinJ(x)  (int)((double)height*(0.8-AmpX*(x)))
#define WinX(i)  (double)(((double)i/width-0.4)/AmpV)
#define WinY(j)  (double)((0.8-(double)j/height)/AmpX)


double         AmpV=0.1,AmpX=0.5;
int            height=300,width=600,marker=1,nmax=2000,slow=100;
Plane_Window   Win;
Control_Window CW,SubCW(Win);
void           draw();
void           startpoint(int);
void           nullcline();
void           explain(int)
{  if(!fork()){ execlp("evince","evince","README_FrictionalMotion.pdf",NULL);}  }


/************************************************************
    Frictional Motion 
*************************************************************/

void derivs(double x,double y[],double dydx[]){
    double u=y[1],v=y[2];
    dydx[1]=v;
    dydx[2]=-u+f(V-v);
}


/************************************************************
    $B%a%$%s%k!<%A%s(B
*************************************************************/

int main(int argc,char *argv[])
{
    int nvar=2;  // $BJ}Dx<0$NJQ?t$N?t(B
    int nok,nbad;
    double x1,x2,eps,h1,hmin,hmax;
    double *ystart=vector(1,nvar);
    
    /*----- $B%&%$%s%I%&$N=i4|@_Dj(B ---------*/

    CW.realtime=0;
    f_dialog d(CW,explain);
    d.resize(2*d.width(),d.height());
    d.string(20);
    d.title(" readme (Click here)");
    CW.realtime=1;
    
    CW<<PARAMETER<<"V"<<V<<b<<"b"<<c<<"c"<<"AmpV"<<AmpV<<"AmpX"<<AmpX
      <<"nmax"<<nmax<<"slow"<<slow;
    
    Win.geometry(width,height)
	<<"Frictional motion(crick this window! )"<<startpoint;
    SubCW<<"x0"<<xx<<"y0"<<yy;
    
    initialize();
    nullcline();          

    /*----- $B%Q%i%a!<%?$,JQ99$5$l$k$?$S$K2r$-D>$9!#(B ---------*/

    while(1){
	
      ystart[2]=xx;  // $B=i4|>r7o(B
      ystart[1]=yy;

      x1=0;          // $B=i4|;~9o(B
      x2=1000;       // $B=*N;;~9o(B
      eps=0.00001;   // $B@:EY(B
      h1=0.01;       // $B=i4|$N9o$_I}(B
      hmin=0.0;      // $B9o$_I}$N:G>.CM(B
      hmax=0.1;      // $B9o$_I}$N:GBgCM(B
      kmax=10000000; // $BG[Ns$NBg$-$5(B
      dxsav=0.01;    
	
      xp=vector(1,kmax);
      yp=matrix(1,nvar,1,kmax);

      // $BE,1~9o$_I}@)8f$D$-$N%k%s%2%/%C%?K!(B
      odeint(ystart,nvar,x1,x2,eps,h1,hmin,hmax,&nok,&nbad,derivs,rkqs);

      // $BIA2h(B
      draw();

      // $B%$%Y%s%HH/@8$r8!=P$9$k$^$GBT$D!#(B
      Win.chev=0; CW.chev=0;
      while(!Win.chev&&!CW.chev) checkevent();
      nullcline();      
      CW.draw();


      free_vector(xp,1,kmax);
      free_matrix(yp,1,nvar,1,kmax);
    }

    free_vector(ystart,1,nvar);
}



/************************************************************
     $BIA2h(B
*************************************************************/

void draw_mark(int,int,int,int);
void erase_mark(int,int,int,int);
struct timespec treq;
long   usec=1E3;
	
void draw()
// $B7k2L$NI=<((B 
{
   static int initial=1;
   int    i0,j0,i,j;

   if(initial){ // $B:BI8<4(B
       Win.draw_line(2,WinI(0.0),0,WinI(0.0),height); 
       Win.draw_line(2,0,WinJ(0.0),width,WinJ(0.0));
       initial=0;
   }

   Win.clear(0); 
   Win.clear(1);
   
   i0=WinI(yp[2][1]); j0=WinJ(yp[1][1]);
   for(int n=2; n<kount; n++){ 
       checkevent();
       if(marker && n>nmax) break;       
       i=WinI(yp[2][n]); j=WinJ(yp[1][n]);
       if(marker){ // $B%H%l!<%5!<IU$-(B
  	   treq.tv_nsec=usec*slow;  nanosleep(&treq,NULL);  // $B<B9T$rCY$i$;$k(B
	   erase_mark(0,i0,j0,5);
	   draw_mark(0,i,j,5);
       }	   
       Win.draw_line(1,i0,j0,i,j);    // $B50@W$NIA2h(B
       i0=i; j0=j;
       XFlush(Win.disp());
   }
}    


void nullcline()
// $B%L%k%/%i%$%s$NIA2h(B
{
   double v;
   int j1=0,j,i0=0;
   Win.clear(4);   
   for(int i=1; i<width; i++){
       v=WinX(i);
       j=WinJ(f(V-v)); Win.draw_line(4,i0,j1,i,j); j1=j; 
       i0=i;
   }       
}    


void draw_mark(int c,int x,int y,int r)
// $B%&%#%s%I%&(B Win $B$K?'(B c $B$G(B $BCf?4(B(i,j) $BH>7B(B r $B$NEI$j$D$V$5$l$?1_$r=q$/!#(B
{
    for(int i=-r; i<r+1; i++)
	for(int j=-r; j<r+1; j++)
	    if(i*i+j*j<=r*r) Win.draw_point(c,i+x,j+y);
}


void erase_mark(int c,int x,int y,int r)
// $B%&%#%s%I%&(B Win $B$K?'(B c $B$N(B $BCf?4(B(i,j) $BH>7B(B r $B$NEI$j$D$V$5$l$?1_$r>C$9!#(B
{    
    for(int i=-r; i<r+1; i++)
	for(int j=-r; j<r+1; j++)
	    if(i*i+j*j<=r*r) Win.erase_point(c,i+x,j+y);
}    


void   startpoint(int n)
// $B%^%&%9$,2!$5$l$?0LCV$N:BI8$r=i4|E@$H$9$k!#$^$?!"1&%\%?%s$,2!$5$l$?$i!"(B
// $BI=<(%b!<%I$rJQ99$9$k!#(B
{
    xx=WinX(Win.ev.xbutton.x);
    yy=WinY(Win.ev.xbutton.y);
    if(n==3) marker=!marker; 
}    
