/*==========================================================================
    global coupled complex Ginzburg Landau equation
      dwdt =c0*w-c2*norm(w)*w+K*c1*(<w>-w)

      <w> = average;
      usually, c0=1,real(c1)=real(c2)=1.0;
===========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Complex_Control.h>
#include <XRS/Graphic.h>
#include <XRS/Plane.h>
using namespace XRS;

#define  Random   drand48()
#define  Color    6


/************** 大域変数と関数宣言 ********************/

double    c1,c2,c02,K;                    // パラメータ
Complex   *w;                             // 場
int       N;                              // システムサイズ
double    t;                              // 時間

void            initial(int);


/*---------------- 以下 XWindow 用 --------------------*/

Complex_Control_Window CW;
Plane_Window    PW;
int             PWsize=200;
int             Initial=0,Trace=1;
double          Amp=0.8;

void            draw();
void            trace(int){ Trace=!Trace; }
void            explain(int)
{  if(!fork()){ execlp("evince","evince","README_GCGL.pdf",NULL);}  }


int main(int argc,char **argv)
{
   double    DT;
   /*--------------- initialize ---------------------*/

   int ar=0;
   ar++; if(argc>ar) N =atoi(argv[ar]);  else N=500;
   ar++; if(argc>ar) DT=atof(argv[ar]);  else DT=0.02;
   ar++; if(argc>ar) c1=atof(argv[ar]);  else c1=2;
   ar++; if(argc>ar) c2=atof(argv[ar]);  else c2=-2;
   ar++; if(argc>ar) K =atof(argv[ar]);  else K=0.5;   
   ar++; if(argc>ar) c02=atof(argv[ar]); else c02=0.4;   

   fprintf(stderr,"usage%% GCGL [N] [DT] [c1] [c2] [K] [c02]\n"
          " current systemsize: N=%d\n"
          " current parameter : c1=%g c2=%g K=%g c0-c2=%g\n"
	  " space and time meshsize: DT=%g\n"
   	  ,N,c1,c2,K,c02,DT); 

   PWsize=400;
   w  = new Complex [N];   
   Complex *w2 = new Complex [N];   

   /*---------------- setup window -------------------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;
   
   CW<<t<<"t"<<initial<<"initial"<<PARAMETER
     << c1 << c2 << K << c02 << Amp 
     <<"c1"<<"c2"<<"K"<<"c02"<<"Amp";
   
   PW.geometry(PWsize,PWsize)<<"complex plane"<<trace;
   
   initialize();
   
   /*--------------- main routine -------------------*/

   initial(0);

   while(1){
      checkevent();

      Complex t0=Complex(1.0,c02+c2)*DT;
      Complex t1=Complex(1.0,c2)*DT;
      Complex t2=Complex(1.0,c1)*K*DT;

      Complex W=0; for(int i=0; i<N; i++) W+=w[i];  W/=N;
      for(int i=0; i<N; i++) w2[i]=w[i]+(t0-t1*norm(w[i]))*w[i]+t2*(W-w[i]);
      for(int i=0; i<N; i++) w[i]=w2[i];

      t+=DT;
      draw();
   }
}


/**********************  初期条件 *************************/

void initial(int)
{  
   for(int i=0; i<N; i++)
      w[i]=polar((double)Random,(double)(2*M_PI*Random));

   t=0; Initial=1;
}


/********************** 書く *************************/

#define PWinI(x) ((int)(PWsize*0.5*(Amp*(x)+1.0)))
#define PWinJ(x) ((int)(PWsize*0.5*(-Amp*(x)+1.0)))		  
		  
void draw(void)
{
    /*======= complex plane snapshot =========*/

    if(Initial){
	PW.clear();
	PW.draw_line(1,PWinI(0),0,PWinI(0),PWsize);
	PW.draw_line(1,0,PWinJ(0),PWsize,PWinJ(0));	
    }
    
    if(Trace) PW.clear(0);
    for(int i=0; i<N; i++){
       PW.draw_point(0,PWinI(real(w[i])),PWinJ(imag(w[i])));
    }

    PW.clear(2);
    PW.draw_circle(2,PWinI(real(w[0])),PWinJ(imag(w[0])),4);

    if(Initial) Initial=0; 
}


