//2次元の拡散

#include <iostream>
#include <iomanip>
#include <cmath>
#include <string>
#include <stdlib.h>  // argc,argvを使うために必要。
using namespace std;

//gnuplot を実行し、データソケットを作成
#include <XRS/Array.h>
#include <XRS/Exec.h>
#include <XRS/DataSocket.h>
using namespace XRS;

Exec gnuplot("gnuplot","gnuplot",NULL,"dev/null");
DataSocket S("diffusion2_socket");

//サブルーチンの宣言 
void arguments(int,char**);  //パラメータの値を決める。
void initial(double **);     //初期条件を設定
void boundary(double **);    //境界条件を設定
void draw(double **);        //表示
void test_output(double **); //テスト出力

//共通で使うパラメータや変数(大域変数)
double L,t,tmax;   //システムサイズ,時間,終了時間
double dt,dx,ram;  //時間刻み、空間刻み, dt/(dx*dx)
int    I;          //要素数
int    dispstep=5; //dispstep 回に1回表示する。


int main(int argc,char **argv)
{
  arguments(argc,argv); //パラメータの値を決める。

  double **u,**u2;
  XRS::array(u,I+2,I+2);
  XRS::array(u2,I+2,I+2);

  initial(u);  //初期条件 

  gnuplot<<"set zr [0:0.1]"<<endl;
  gnuplot<<"set ticslevel 0"<<endl;
  gnuplot<<"set style data lines"<<endl;
  gnuplot<<"set pm3d"<<endl;
  gnuplot<<"set hidden3d"<<endl;

  while(t<tmax){
    for(int k=0; k<dispstep; k++){ 

      for(int i=1; i<I+1; i++)
        for(int j=1; j<I+1; j++)
  	  u2[i][j]=u[i][j]+ram*(u[i-1][j]+u[i+1][j]+u[i][j-1]+u[i][j+1]-4.0*u[i][j]);  //計算
      

      for(int i=1; i<I+1; i++)
        for(int j=1; j<I+1; j++) u[i][j]=u2[i][j];     //更新

      boundary(u);  //境界条件
      t+=dt;     //時間をdt進める。
    }
    draw(u); //表示
  }
  
  return 0;
}


//******** パラメータを決める *******
void arguments(int argc,char** argv)
{
  //コマンドライン引数があればその値を使う。なければ既定値を代入。
  if(argc>1) tmax=atof(argv[1]); else tmax=10;
  if(argc>2) L   =atof(argv[2]); else L=20.0;
  if(argc>3) dt  =atof(argv[3]); else dt=0.02;
  if(argc>4) dx  =atof(argv[4]); else dx=0.5;

  I=int(L/dx+0.5); //要素数を決める(四捨五入)  
  ram=dt/(dx*dx);

  //使う値を表示。
  cout<< setprecision(15);
  cerr<< setprecision(15);                    
  cerr<<"tmax="<<tmax<<", L="  <<L   <<endl;  //<==ちょっと修正。標準エラー出力にして、
  cerr<<"dt="  <<dt  <<", dx=" <<dx  << endl; //標準出力と使い分けると便利です。
  cerr<<"I="   <<I   <<", ram="<<ram << endl; //パイプ(>など)を使うと、必要に応じて
                                              //それぞれ別のファイルに書き込めます。
}

//*********** 初期条件 **************
void initial(double **u)
{
  for(int i=1;i<I+1;i++)
    for(int j=1;j<I+1;j++){
      double x=i-I/2, y=j-I/2;
      u[i][j]=0.1*exp(-(x*x+y*y)/4.0);
    } 

  //u[I/2][I/2]=1.0;
  boundary(u);
}

//***********  境界条件 *************
void boundary(double **u)
{
  for(int i=1; i<I+1; i++){
    u[i][0] = u[i][I];  u[i][I+1] = u[i][1];}
  for(int j=1; j<I+1; j++){
    u[0][j] = u[I][j];  u[I+1][j] = u[1][j];}  //周期
}

//************ 表示 *****************
void draw(double **u)
{
  gnuplot<<"splot '< datasocket diffusion2_socket'"<<endl; 
  S.open();        
  for(int i=1; i<I+1; i++){
    for(int j=1; j<I+1; j++){
      S<<u[i][j]<<endl;
    }
    S<<endl;
  }
  S.close();
}

