/*==========================================================================
  excitable system. 
  It's behavior for large diffusion constant was shown by Ryo Kobayashi.   

     ∂t u = f(u) - v + ∂xx u
     ∂t v = u + D * ∂xx v
     f(u)=-u+0.5*(tanh((u-a)/δ)+tanh(a/δ))
==========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Graphic.h>
#include <XRS/Plane.h>
using namespace XRS;

#define f(u)        (-u+0.5*(tanh((u-a)/delta)+tanh(a/delta)))
#define integer(x)  ((int)(floor(x)+floor(2*((x)-floor(x)))))


/************** 大域変数と関数宣言 ********************/

double          a,D,delta;
double          *u,*v;
double          L;
double          t;

void            initial(int);

/*---------------- 以下 XWindow 用 --------------------*/

Control_Window  CW;
Graphic_Window  GW;
Plane_Window    W,PW;
double          depth=1.5;
int             Wimax,Wjmax,PWsize=200,GWjmax=200,Dispstep=500,Clear=1;
int             Initial=0;

void            draw();
void            PW_draw(int);
void            GW_draw();
void            explain(int)
{  if(!fork()){ execlp("evince","evince","README_excitable_1d.pdf",NULL);}  }
void            show(int){ Clear=!Clear; }


int main(int argc,char **argv)
{
   double    DT,DX;

   /*--------------- initialize ---------------------*/

   int ar=0;
   ar++; if(argc>ar) L=atof(argv[ar]);      else L=400;
   ar++; if(argc>ar) DX=atof(argv[ar]);     else DX=0.5;
   ar++; if(argc>ar) DT=atof(argv[ar]);     else DT=0.01;
   ar++; if(argc>ar) a=atof(argv[ar]);      else a=0.1;
   ar++; if(argc>ar) D=atof(argv[ar]);      else D=1.4;
   ar++; if(argc>ar) delta=atof(argv[ar]);  else delta=0.02;   

   fprintf(stderr,"usage%% excitable [L] [DX] [DT] [a] [D] [delta]\n"
          " current systemsize: L=%g\n"
          " current parameter : a=%g D=%g delta=%g\n"
	  " space and time meshsize: DX=%g DT=%g\n"
   	  ,L,a,D,delta,DX,DT); 

   Wimax=(int)(L/DX); Wjmax=200; 
   u  = new double [Wimax];      v  = new double [Wimax];
   double   *u2,*v2;
   u2 = new double [Wimax];      v2 = new double [Wimax];   

   /*---------------- setup window -------------------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   
       f_dialog d2(CW,show);
       d2.resize(2*d2.width(),d2.height());
       d2.string(20);
       d2.title(" the way to showing ");
   CW.realtime=1;
   
   CW<<t<<"t"<<PARAMETER
     << a << D << delta << Dispstep << depth
     <<"a"<<"D"<<"delta"<<"slow"    << "depth";
   
   W.geometry(Wimax,Wjmax,1)<<initial
       <<"excitable system( A mouse click sets up the initial condition. )";

   GW.geometry(Wimax,GWjmax)<<"space-time series"<<MONO();	   
   PW.geometry(PWsize,PWsize,1)<<"u-v";	   
   
   initialize();
   
   /*--------------- main roution -------------------*/

   initial(0);
   int       step=0;   
   
   while(1){
      checkevent();
      if(CW.chev) PW_draw(1);

      double txx=DT/(DX*DX);
      double du,dv;
      int GWstep=Dispstep/10;

      for(int ts=0; ts<Dispstep;ts++){

         for(int i=1;i<Wimax-1;i++){
             du=u[i-1]+u[i+1]-2.0*u[i]; 
             u2[i]=u[i]+txx*du+DT*(f(u[i])-v[i]);
         }

         for(int i=1;i<Wimax-1;i++){
             dv=v[i-1]+v[i+1]-2.0*v[i];
             v2[i]=v[i]+txx*D*dv+DT*u[i];
         }

         for(int i=1;i<Wimax-1;i++){
             v[i]=v2[i];
             u[i]=u2[i];
         }
             
         v[0]=v[1];
         v[Wimax-1]=v[Wimax-2];
         u[0]=u[1];
         u[Wimax-1]=u[Wimax-2];
         if(step>GWstep){ GW_draw(); step=0;} step++;
      }

      t+=DT*Dispstep;      

      draw();
      PW_draw(0);
    }
}


/**********************  初期条件 *************************/

#define WinX(j) ((0.5-(double)(j)/(double)Wjmax)/0.3)
#define WinJ(x) ((int)(Wjmax*(-0.3*(x)+0.5)))		  

void initial(int n)
{
   static int i0=(int)(0.6*(double)Wimax);
   static double up=2.5;
   
   for(int i=0; i<Wimax; i++){ u[i] = v[i]=0.0; }

   if(n){ i0=W.ev.xbutton.x;  up=WinX(W.ev.xbutton.y); W.chev=0; }

   if(i0>=Wimax/2){ for(int i=Wimax-i0; i<i0; i++)  u[i]=up;}
   else           { for(int i=i0; i<Wimax-i0; i++)  u[i]=up;}    
   
   u[0]=u[Wimax-2];
   u[Wimax-1]=u[1];
   v[0]=v[Wimax-2];
   v[Wimax-1]=v[1];

   t=0; Initial=1;
   draw();
   PW_draw(1);
}


/********************** 書く *************************/

void draw(void)
{
    /*======= snapshot =========*/
    int j,j0;
    if(Initial){
	W.clear();
	W.draw_line(3,Wimax/2,0,Wimax/2,Wjmax);	
	W.draw_line(3,0,WinJ(0),Wimax,WinJ(0));
	W.draw_line(2,0,WinJ(1),Wimax,WinJ(1));
	W.draw_line(2,0,WinJ(-1),Wimax,WinJ(-1));
        Initial=0;
    }		 

    if(Clear) W.clear(0);
    j0=WinJ(u[0]);
    for(int i=0; i<Wimax; i++){
       j=WinJ(u[i]);
       W.draw_line(0,i-1,j0,i,j);
       j0=j;
    }   

    if(Clear) W.clear(1);    
    j0=WinJ(v[0]);
    for(int i=0; i<Wimax; i++){
       j=WinJ(v[i]);
       W.draw_line(1,i-1,j0,i,j);
       j0=j;
    }   
}


void GW_draw(void)
{
    static int j;
    
    if(Initial){ GW.clear(); j=GWjmax-1; }
    
    for(int i=0; i<Wimax; i++)
      if(u[i]>0){ GW.draw_point(depth*u[i],i,j);}

    j--;
    if(j<0){ GW.clear(); j=GWjmax-1;}
}


#define PWinX(i)  (((double)(i)/(double)PWsize-0.5)/0.3)
#define PWinI(x)  ((int)(PWsize*(-0.3*(x)+0.5)))		  

void PW_draw(int init)
{
    double x;
    int j0,k0,j,k;

    if(init){
      PW.clear();	
      PW.draw_line(2,PWsize/2,0,PWsize/2,PWsize);	
      PW.draw_line(2,0,PWinI(0),PWsize,PWinI(0));

      x=PWinX(0); j0=PWinI(f(x));
      for(int i=1; i<PWsize; i++){
	x=PWinX(i); j=PWinI(f(x));
	PW.draw_line(1,i-1,j0,i,j);
	j0=j;
      }
    }

    PW.clear(0);
    j0=PWinI(u[0]); k0=PWinI(v[0]);
    for(int i=1; i<Wimax; i+=2){
       j=PWinI(u[i]); k=PWinI(v[i]);
       PW.draw_line(0,j0,k0,j,k);
       j0=j; k0=k;
    }   
}    


